package wobs.bool;

import java.beans.*;
import java.io.*;


import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;


public class BooleanProperties extends WobProperties implements Externalizable, PropertyEditorCollectionFactory {
	
	/**
	 * @exclude
	 */
	static final long serialVersionUID = 3392609606122409387L;
	private boolean value = true;
	private boolean startupPush;
	//private PropertyEditorCollection propertyEditorCollection = null;
	//    private JPanel panel;

	public String toString() {
		if (value)
			return Messages.getString("BooleanProperties.True"); //$NON-NLS-1$
		else
			return Messages.getString("BooleanProperties.False"); //$NON-NLS-1$
	}

	public void writeExternal(ObjectOutput out) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) out;
			super.writeExternal(oos);
			oos.writeBoolean(value);
			oos.writeBoolean(startupPush);
		} catch (Exception e) {
			System.out.println("BooleanDialog: " + e); //$NON-NLS-1$
		}
	}

	public void readExternal(ObjectInput in) {
		try {
			ObjectInputStream ois = (ObjectInputStream) in;
			super.readExternal(ois);
			value = ois.readBoolean();
			if (Project.fileFormatVersion >= 12) {
				startupPush = ois.readBoolean();
			}
		} catch (Exception e) {
			System.out.println("BooleanDialog: " + e); //$NON-NLS-1$
			if (!e.getMessage().equals("progress")) //$NON-NLS-1$
				e.printStackTrace();
		}
	}



	public void setValue(boolean state) {
		if (this.value != state) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.value, state) {
				public void setValue(boolean value) {
					BooleanProperties.this.setValue(value);
				}
			};
			getUndoManager().addEdit(edit);				
			
			this.value = state;
			firePropertyChange("booleanValueChanged", null, new Boolean(value));//Integer((value) ? 1 : 0)); //$NON-NLS-1$
		}
	}

	public boolean getValue() {
		return value;
	}

	public void setPushedAtStartup(boolean startupPush) {
		if (this.startupPush != startupPush) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.startupPush, startupPush) {
				public void setValue(boolean value) {
					setPushedAtStartup(value);
				}
			};
			getUndoManager().addEdit(edit);					
			
			this.startupPush = startupPush;
			firePropertyChange("startupPushChanged", null, new Boolean(startupPush)); //$NON-NLS-1$
		}
	}

	public boolean isPushedAtStartup() {
		return startupPush;
	}

	public PropertyEditorCollection createPropertyEditorCollection() {
	//System.out.println(1);
	//nanoxml.XMLElement e = new nanoxml.XMLElement();
		
		//if (propertyEditorCollection != null)
		//	System.out.println("Error in NumberProperties.createPropertyEditorList()");

		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(this);

		// Create Wob name property editor
		/*ComboBoxPropertyEditor valueEditor = new ComboBoxPropertyEditor("booleanValueChanged", "Value", new String[] {"False (No)", "True (Yes)"}, (value) ? 1 : 0);
		valueEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				int index = ((Integer) evt.getNewValue()).intValue();
				setValue(index==1);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueEditor);*/
		BooleanPropertyEditor valueEditor = new BooleanPropertyEditor("booleanValueChanged", Messages.getString("BooleanProperties.Value"), getValue()); //$NON-NLS-1$ //$NON-NLS-2$
		valueEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setValue(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueEditor);


		BooleanPropertyEditor startupPushEditor = new BooleanPropertyEditor("startupPushChanged", Messages.getString("BooleanProperties.Push_at_Startup"), isPushedAtStartup()); //$NON-NLS-1$ //$NON-NLS-2$
		startupPushEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setPushedAtStartup(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(startupPushEditor);

		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

	/*public void destroyPropertyEditorCollection(PropertyEditorCollection propertyEditorCollection) {
		removeNamedChangeListener(propertyEditorCollection);
	}*/


	/*public WobPropertiesPanel createWobPropertiesPanel() {
		return new BooleanPropertiesPanel();
	}*/
	/* panel = new JPanel();
	 panel.setLayout(new GridBagLayout());
	 GridBagConstraints gbc = new GridBagConstraints();
	 gbc.insets = new Insets(5,5,5,5);
	 gbc.weightx = 1;
	 gbc.weighty = 1;
	
	 gbc.gridx = 0;
	 gbc.gridy = 0;
	//        gbc.anchor=gbc.EAST;
	 gbc.fill=gbc.HORIZONTAL;
	 JPanel jp=new JPanel(new FlowLayout());
	 jp.setBorder(new TitledBorder(new EtchedBorder(),"Value"));
	//        panel.add(new JLabel("Value:"),gbc);
	
	 trueButton = new JRadioButton("True (Yes)");
	 falseButton = new JRadioButton("False (No)");
	 ButtonGroup bg = new ButtonGroup();
	 bg.add(trueButton);
	 bg.add(falseButton);
	 jp.add(trueButton);
	
	 jp.add(falseButton);
	 panel.add(jp,gbc);
	
	 gbc.gridx = 0;
	 gbc.gridy = 1;
	//        gbc.gridwidth=4;
	
	 startupBox=new JCheckBox("Trigger \"Boolean Set\" outport at Startup",startupSet);
	 panel.add(startupBox,gbc);
	// Nedanst?ende fungerar ej, false ej satt n?r den borde vara det om man ?ppnar dialogen. JDK 1.3.0-bug?
	//        falseButton.setSelected(state);
	//        trueButton.setSelected(state);
	
	 if (state) trueButton.setSelected(true);
	 else falseButton.setSelected(true);
	
	 return panel;
	}*/

	/* public void destroyGUI() {
	     panel.removeAll();
	     panel = null;
	     trueButton = null;
	     falseButton = null;
	     startupBox=null;
	 }*/

	/* public boolean okPressed() {
	
	     return super.okPressed();
	 }*/

	public BooleanProperties() {
		super(); // S? att parentBVis blir satt
	}

	public void writeData(DataOutputStream dos) {
		try {
			dos.writeBoolean(value);
			dos.writeBoolean(startupPush);
		} catch (IOException e) {
			System.out.println(e);
		}
	}

}