package wobs.color;

import java.awt.*;
import java.beans.*;
import java.io.*;


import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;
import com.wirefusion.utils.*;


public class ColorProperties extends WobProperties implements Externalizable, PropertyEditorCollectionFactory {
	static final long serialVersionUID = 6232503898946401021L;
	private boolean startupPush;
	private Color color = Color.black;

	public void writeExternal(ObjectOutput out) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) out;
			super.writeExternal(oos);
			oos.writeInt(color.getRGB());
			oos.writeBoolean(startupPush);
		} catch (Exception e) {
			System.out.println("ColorDialog: " + e); //$NON-NLS-1$
		}
	}

	public void readExternal(ObjectInput in) {
		try {
			ObjectInputStream ois = (ObjectInputStream) in;
			super.readExternal(ois);
			color = new Color(ois.readInt());
			if (Project.fileFormatVersion >= 12) {
				startupPush = ois.readBoolean();
			}
		} catch (Exception e) {
			System.out.println("ColorDialog: " + e); //$NON-NLS-1$
			if (!e.getMessage().equals("progress")) //$NON-NLS-1$
				e.printStackTrace();
		}
	}

	/*public WobPropertiesPanel createWobPropertiesPanel() {
	    return new ColorPropertiesPanel();
	}*/



	public String toString() {
	  String template = Messages.getString("ColorProperties.RED_GREEN_BLUE_string"); //$NON-NLS-1$
	  return WFUtils.format(template, new Object[] {String.valueOf(color.getRed()), String.valueOf(color.getGreen()), String.valueOf(color.getBlue())});
	}

	public void setPushedAtStartup(boolean startupPush) {
		if (this.startupPush != startupPush) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.startupPush, startupPush) {
				public void setValue(boolean value) {
					setPushedAtStartup(value);
				}
			};
			getUndoManager().addEdit(edit);				
			
			this.startupPush = startupPush;
			firePropertyChange("startupPushChanged", null, new Boolean(startupPush)); //$NON-NLS-1$
		}
	}

	public boolean isPushedAtStartup() {
		return startupPush;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		if (!this.color.equals(color)) {
			UndoableColorEdit edit = new UndoableColorEdit(this.color, color) {
				public void setValue(Color value) {
					setColor(value);
				}
			};
			getUndoManager().addEdit(edit);				
			
			
			this.color = color;
			firePropertyChange("colorValueChanged", null, color); //$NON-NLS-1$
		}
	}

	public PropertyEditorCollection createPropertyEditorCollection() {
		//if (propertyEditorCollection != null)
		//	System.out.println("Error in ColorProperties.createPropertyEditorList()");

		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(this);

		// Create Wob name property editor
		ColorPropertyEditor valueEditor = new ColorPropertyEditor("colorValueChanged", Messages.getString("ColorProperties.Color"), color); //$NON-NLS-1$ //$NON-NLS-2$
		valueEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				Color colorValue = ((Color) evt.getNewValue());
				setColor(colorValue);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueEditor);

		BooleanPropertyEditor startupPushEditor = new BooleanPropertyEditor("startupPushChanged", Messages.getString("ColorProperties.Push_at_Startup"), isPushedAtStartup()); //$NON-NLS-1$ //$NON-NLS-2$
		startupPushEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setPushedAtStartup(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(startupPushEditor);

		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

	/*public void destroyPropertyEditorCollection(PropertyEditorCollection propertyEditorCollection) {
		removeNamedChangeListener(propertyEditorCollection);
	}*/

	public ColorProperties() {
		super(); // S? att parentBVis blir satt
	}

	public void writeData(DataOutputStream dos) {
		try {
			dos.writeInt(color.getRGB());
			dos.writeBoolean(startupPush);
		} catch (IOException e) {
			System.out.println(e);
		}
	}

}