/*
 * @(#)ColorWob.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.color.player;

import java.io.*;
import com.wirefusion.player.*;

/**
 * Represents a Color object in the WireFusion player.
 */

public class ColorWob extends Wob {
  /**
   * Contains the Color value (RRGGBB)
   * @exclude
   */
  public int _color;
  
  private boolean pushAtStartup = false;

  /**
   * @exclude
   */
  public ColorWob() {  	
  }

  /**
   * Creates a ColorWob object
   * 
   * @param color an RGB value
   */
  public ColorWob(int color) {
  	this._color = color;
  }

	/**
	 * Returns the color value (RGB).
	 * 
	 * @return the color value
	 */
	public int getValue() {
		return _color;
	}
  
  /**
   * @exclude
   */
  public void _presentationStarted() {
    if (pushAtStartup)
    	pushValue();
	}
  
	/**
	 * Pushes the color value through the Value Pushed out-port
	 * @exclude
	 */	
  private void pushValue() {
    sendColor(_color, 1);
    sendText("#" + Long.toString(_color&((long)0xFFFFFF), 16), 2);       
  }
  

/**
 * 
 */
  public void processInportEvent(Wob arg, int inport) {
    switch (inport) {
    case 1: // Set Value
      _color = ((ColorWob) arg)._color;
      break;
    case 2: // Push Value
      pushValue();
      break;
    case 3: // Set color component
    case 4:
    case 5:
      int up = (5 - inport) * 8;
      _color &= 0xffffff - (0xff << up);
      double v = ((wobs.number.player.NumberWob) arg)._value;
      _color += (int) Math.min(255, Math.max(v, 0)) << up;
      break;
    case 6: // Set color from Hex String
      try {
        String hexString = ((wobs.text.player.TextWob) arg)._value;
        if (hexString.startsWith("#"))
          hexString = hexString.substring(1);
        _color = Integer.parseInt(hexString, 16);        
      } catch (Exception e) {
        e.printStackTrace();
      }
      break;
    }
  }
  
  /**
   * @exclude
   */
  public void readData(DataInputStream dis) {
    try {
      _color = dis.readInt();
      pushAtStartup = dis.readBoolean();
    } catch (Exception e) {
    }
  }
}