package wobs.number;

import java.beans.*;
import java.io.*;


import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;
import com.wirefusion.utils.*;











public class NumberProperties extends WobProperties implements Externalizable, PropertyEditorCollectionFactory { //, ActionListener {
	static final long serialVersionUID = 9118093502966213100L;
	private double value;
	private boolean startupPush;

	public NumberProperties() {	  
		super();
	}
	
	public void writeExternal(ObjectOutput out) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) out;
			super.writeExternal(oos);
			oos.writeDouble(value);
			oos.writeBoolean(startupPush);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void readExternal(ObjectInput in) {
		try {
			ObjectInputStream ois = (ObjectInputStream) in;
			super.readExternal(ois);
			value = ois.readDouble();
			if (Project.fileFormatVersion >= 12) {
				startupPush = ois.readBoolean();
			}
		} catch (Exception e) {
			System.out.println("NumberDialog: " + e);  //$NON-NLS-1$
			if (!e.getMessage().equals("progress")) //$NON-NLS-1$
				e.printStackTrace();
		}
	}

	// No longer needed...handled automatically by NumberTextField.setText()
	public String toString() {
		return WFUtils.getNumber(""+getValue()); //$NON-NLS-1$
	}

	public void setValue(double value) {
		if (this.value != value) {
			UndoableDoubleEdit edit = new UndoableDoubleEdit(this.value, value) {
				public void setValue(double value) {
					NumberProperties.this.setValue(value);
				}
			};
			getUndoManager().addEdit(edit);				
			
			this.value = value;
			firePropertyChange("numberValueChanged", null, new Double(value)); //$NON-NLS-1$
		}
	}

	public double getValue() {
		return value;
	}

	public void setPushedAtStartup(boolean startupPush) {
		if (this.startupPush != startupPush) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.startupPush, startupPush) {
				public void setValue(boolean value) {
					setPushedAtStartup(value);
				}
			};
			getUndoManager().addEdit(edit);			
			
			this.startupPush = startupPush;
			firePropertyChange("startupPushChanged", null, new Boolean(startupPush)); //$NON-NLS-1$
		}
	}

	public boolean isPushedAtStartup() {
		return startupPush;
	}

	public PropertyEditorCollection createPropertyEditorCollection() {
		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(this);
		
		NumberPropertyEditor valueEditor = new NumberPropertyEditor("numberValueChanged", Messages.getString("NumberProperties.Value"), new Double(getValue()));   //$NON-NLS-1$//$NON-NLS-2$
		valueEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {			
				double numberValue = ((Double) evt.getNewValue()).doubleValue();
				setValue(numberValue);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueEditor);

		BooleanPropertyEditor startupPushEditor = new BooleanPropertyEditor("startupPushChanged", Messages.getString("NumberProperties.Push_at_Startup"), isPushedAtStartup()); //$NON-NLS-1$ //$NON-NLS-2$
		startupPushEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setPushedAtStartup(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(startupPushEditor);

		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

	public void writeData(DataOutputStream dos) {
		try {
			dos.writeDouble(getValue());
			dos.writeBoolean(isPushedAtStartup());
		} catch (IOException e) {
			System.out.println(":" + e); //$NON-NLS-1$
		}
	}
}