package wobs.number2d;

import java.beans.*;
import java.io.*;


import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;
import com.wirefusion.utils.*;

public class TwoDNumberProperties extends WobProperties implements Externalizable, PropertyEditorCollectionFactory { //, ActionListener {
	static final long serialVersionUID = 4706859416618107347L;
	private double valueX;
	private double valueY;
	private boolean startupPush;

	public void writeExternal(ObjectOutput out) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) out;
			super.writeExternal(oos);
			oos.writeDouble(valueX);
			oos.writeDouble(valueY);
			oos.writeBoolean(startupPush);
		} catch (Exception e) {
			System.out.println("2D Number Dialog: " + e); //$NON-NLS-1$
		}
	}

	public void readExternal(ObjectInput in) {
		try {
			ObjectInputStream ois = (ObjectInputStream) in;
			super.readExternal(ois);
			valueX = ois.readDouble();
			valueY = ois.readDouble();
			if (Project.fileFormatVersion >= 12) {
				startupPush = ois.readBoolean();
			}
		} catch (Exception e) {
			System.out.println("2D Number Dialog: " + e); //$NON-NLS-1$
			if (!e.getMessage().equals("progress")) //$NON-NLS-1$
				e.printStackTrace();
		}
	}

	public String toString() {
		String strX = "" + valueX; //$NON-NLS-1$
		if (strX.endsWith(".0")) //$NON-NLS-1$
			strX = strX.substring(0, strX.length() - 2);
		String strY = "" + valueY; //$NON-NLS-1$
		if (strY.endsWith(".0")) //$NON-NLS-1$
			strY = strY.substring(0, strY.length() - 2);
		String template = Messages.getString("TwoDNumberProperties.position_string"); //$NON-NLS-1$
		return WFUtils.format(template, new Object[] {strX, strY});

	}

	public void setValueX(double value) {
		if (this.valueX != value) {
			UndoableDoubleEdit edit = new UndoableDoubleEdit(this.valueX, value) {
				public void setValue(double value) {
					TwoDNumberProperties.this.setValueX(value);
				}
			};
			getUndoManager().addEdit(edit);			
			
			this.valueX = value;
			firePropertyChange("numberValueXChanged", null, new Double(value)); //$NON-NLS-1$
		}
	}

	public void setValueY(double value) {
		if (this.valueY != value) {
			UndoableDoubleEdit edit = new UndoableDoubleEdit(this.valueY, value) {
				public void setValue(double value) {
					TwoDNumberProperties.this.setValueY(value);
				}
			};
			getUndoManager().addEdit(edit);			
			
			this.valueY = value;
			firePropertyChange("numberValueYChanged", null, new Double(value)); //$NON-NLS-1$
		}
	}

	public void setPushedAtStartup(boolean startupPush) {
		if (this.startupPush != startupPush) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.startupPush, startupPush) {
				public void setValue(boolean value) {
					setPushedAtStartup(value);
				}
			};
			getUndoManager().addEdit(edit);			
			
			this.startupPush = startupPush;
			firePropertyChange("startupPushChanged", null, new Boolean(startupPush)); //$NON-NLS-1$
		}
	}

	public boolean isPushedAtStartup() {
		return startupPush;
	}

	public double getValueX() {
		return valueX;
	}

	public double getValueY() {
		return valueY;
	}

	public PropertyEditorCollection createPropertyEditorCollection() {
		//if (propertyEditorCollection != null)
		//	System.out.println("Error in NumberProperties.createPropertyEditorList()");

		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(this);

		// Create Wob name property editor
		NumberPropertyEditor valueXEditor = new NumberPropertyEditor("numberValueXChanged", Messages.getString("TwoDNumberProperties.X"), new Double(getValueX())); //$NON-NLS-1$ //$NON-NLS-2$
		//valueXEditor.setType(NumberPropertyEditor.DOUBLE);
		valueXEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				double numberValue = ((Double) evt.getNewValue()).doubleValue();
				setValueX(numberValue);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueXEditor);

		// Create Wob name property editor
		NumberPropertyEditor valueYEditor = new NumberPropertyEditor("numberValueYChanged", Messages.getString("TwoDNumberProperties.Y"), new Double(getValueY())); //$NON-NLS-1$ //$NON-NLS-2$
		//valueYEditor.setType(NumberPropertyEditor.DOUBLE);
		valueYEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				double numberValue = ((Double) evt.getNewValue()).doubleValue();
				setValueY(numberValue);
			}
		});
		propertyEditorCollection.addPropertyEditor(valueYEditor);

		BooleanPropertyEditor startupPushEditor = new BooleanPropertyEditor("startupPushChanged", Messages.getString("TwoDNumberProperties.Push_at_Startup"), isPushedAtStartup()); //$NON-NLS-1$ //$NON-NLS-2$
		startupPushEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setPushedAtStartup(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(startupPushEditor);

		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

	/*public void destroyPropertyEditorCollection(PropertyEditorCollection propertyEditorCollection) {
		removeNamedChangeListener(propertyEditorCollection);
		//propertyEditorCollection = null;
	}*/



	/*public boolean okPressed() {
	    if (xField.check() && yField.check()) {
	        valueX = Double.parseDouble(xField.getText().trim());
	        valueY = Double.parseDouble(yField.getText().trim());
	        startupSet=startupBox.isSelected();
	        return super.okPressed();
	    } else return false;
	}*/

	/* public WobPropertiesPanel createWobPropertiesPanel() {
	     return new TwoDNumberPropertiesPanel();
	 }*/

	/*public void destroyGUI() {
	    panel.removeAll();
	    panel = null;
	    xField = null;
	    yField = null;
	    startupBox=null;
	}*/

	public TwoDNumberProperties() {
		super(); // S? att parentBVis blir satt
	}

	public void writeData(DataOutputStream dos) {
		try {
			dos.writeDouble(valueX);
			dos.writeDouble(valueY);
			dos.writeBoolean(startupPush);
		} catch (Exception e) {
			System.out.println(e);
		}
	}

}