package wobs.scene;
import java.beans.*;
import java.io.*;

import javax.swing.*;
import javax.swing.undo.*;


import com.wirefusion.*;
import com.wirefusion.gui.*;
import com.wirefusion.project.*;
import com.wirefusion.propertyeditor.*;
import com.wirefusion.undo.*;

public class SceneWobProperties extends WobProperties implements Externalizable, PropertyEditorCollectionFactory { //, ActionListener {
	static final long serialVersionUID = -1906439122676504900L;
	private boolean pushSceneAtStartup = true;
	//private boolean isVirtualScene = false;	
	
	private int sceneMode = OFFSCREEN_SCENE_MODE;
	public static final int OFFSCREEN_SCENE_MODE = 0;
	public static final int STANDARD_GROUPING_SCENE_MODE = 1;		
	public static final int BUFFERED_GROUPING_SCENE_MODE = 2;		
	private static final int[] targetAreaInPorts = {30002,30003,29004,29014,29015,29016,29017};
	private static final int[] targetAreaOutPorts = {29101,29102,29103,29104,29105,29106,29107,29115,29116,29117, 29120};	
	
	private OperatorAreaModel lastOperatorAreaModel = null; // Backup, som anvnds nr man slagit av och sedan p target arean igen, s att position mm behlls
	//private int lastLayerOrder = -1; // Anvnds tillsammans med lastOperatorAreaModel


	public void writeExternal(ObjectOutput out) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) out;
			super.writeExternal(oos);
			oos.writeBoolean(pushSceneAtStartup);
			oos.writeInt(sceneMode);			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void readExternal(ObjectInput in) {
		try {
			ObjectInputStream ois = (ObjectInputStream) in;
			super.readExternal(ois);
			pushSceneAtStartup = ois.readBoolean();
			if (Project.fileFormatVersion >= 49 && Project.fileFormatVersion < 51) {
				ois.readBoolean();				
			} 
			if (Project.fileFormatVersion >= 51){
				sceneMode = ois.readInt();
			} else {
				boolean isOffscreen = (_getWobModel().getOperatorAreaModel() == null);
				if (isOffscreen)
					sceneMode = OFFSCREEN_SCENE_MODE;
				else
					sceneMode = BUFFERED_GROUPING_SCENE_MODE;	
			}
		} catch (Exception e) {
			e.printStackTrace();
			if (!e.getMessage().equals("progress")) //$NON-NLS-1$
				e.printStackTrace();
		}
	}

	public boolean _dropped() {
		configureSceneMode();
		return true;
	}
	
	public void configureSceneMode() {		
		if ((_getWobModel().getParent() != null) && (_getWobModel().getParent().getType() == WobModel.SCENE)) {
			//WF.mainFrame.repaint();
			int returnValue = JOptionPane.showConfirmDialog(WF.mainFrame, Messages.getString("SceneWobProperties.Do_you_want_the_Scene_to_be_in_grouping_mode"), Messages.getString("SceneWobProperties.Scene_mode"), JOptionPane.YES_NO_OPTION); //$NON-NLS-1$ //$NON-NLS-2$
			boolean groupingScene = (returnValue == JOptionPane.YES_OPTION);
			if (groupingScene) {
				setSceneMode(STANDARD_GROUPING_SCENE_MODE, true);
			} else {
				setSceneMode(OFFSCREEN_SCENE_MODE, true);
			}
		} else {
			setSceneMode(OFFSCREEN_SCENE_MODE, true);			
		}
	}
	
	private void removePort(int portNo, boolean isInport) {
		PortBundle portBundle = (isInport) ? _getWobModel().getInPorts() : _getWobModel().getOutPorts();
		/*if (calledDuringInit) {
			portBundle.deletePortWithoutUndo(portNo);			
		} else {*/
			Port port = portBundle.getOriginalPort(portNo);
			if (port != null)
				port.delete();			
		//}
	}
	
	private void createPort(int portNo, boolean isInport) {
		PortBundle portBundle = (isInport) ? _getWobModel().getInPorts() : _getWobModel().getOutPorts();	
		Port aPort = portBundle.getOriginalPort(portNo);
		if (aPort == null) { // Om den inte fanns redan		
//System.out.println(isInport+":"+portNo);			
			if (isInport)
				aPort =	WobHandler.getFastAccessData("scene").getInportConfig(portNo).createPort(_getWobModel(), null);
			else
				aPort =	WobHandler.getFastAccessData("scene").getOutportConfig(portNo).createPort(_getWobModel(), null);		
	    portBundle.addOriginalPort(aPort);
		}
	}
	
	private void updateSceneMode() {
		//boolean wiresWillBeRemoved = false;
		if (sceneMode == OFFSCREEN_SCENE_MODE) {
			for (int i = 0; i<targetAreaInPorts.length; i++)
				removePort(targetAreaInPorts[i], true);
			for (int i = 0; i<targetAreaOutPorts.length; i++)
				removePort(targetAreaOutPorts[i], false);
			removePort(30001, true); // set opacity
			removePort(29013, true); // stamp bg
			createPort(0, true);
			createPort(0, false);	
			//if (!calledDuringInit)
			lastOperatorAreaModel = _getWobModel().getOperatorAreaModel();
			/*if (getWobModel().getParent() != null) // ondigt?
				lastLayerOrder = getWobModel().getParent().getChildrenLayerOrder().indexOf(this);*/
			_getWobModel().removeOperatorAreaModel();
			//getWobModel().getOperatorAreaModel().delete();	
		} else {		
			if (_getWobModel().getOperatorAreaModel() == null) {		
				OperatorAreaModel operatorAreaModel = null;
				if (lastOperatorAreaModel == null) {
					operatorAreaModel = new OperatorAreaModel();
				} else {
					operatorAreaModel = lastOperatorAreaModel;
				}
				
				_getWobModel().setOperatorAreaModel(operatorAreaModel, true);
				/*if (lastLayerOrder != -1) {
					getWobModel().getParent().getChildrenLayerOrder().remove(operatorAreaModel);
					lastLayerOrder = Math.min(getWobModel().getParent().getChildrenLayerOrder().size(), lastLayerOrder);
					getWobModel().getParent().getChildrenLayerOrder().insertElementAt(operatorAreaModel, lastLayerOrder);
				}*/

				_getWobModel().getParent().getChildrenLayerOrder().remove(this);
				operatorAreaModel.setToPixelChanger(true);
				operatorAreaModel.setVerticallyResizable(false);
				operatorAreaModel.setHorizontallyResizable(false);
			}			
			
			for (int i = 0; i<targetAreaInPorts.length; i++)
				createPort(targetAreaInPorts[i], true);
			for (int i = 0; i<targetAreaOutPorts.length; i++)
				createPort(targetAreaOutPorts[i], false);
			createPort(30001, true);
			createPort(29013, true);
			if (sceneMode == BUFFERED_GROUPING_SCENE_MODE) {
				createPort(0, true);
				createPort(0, false);
			} else {
				removePort(0, true);
				removePort(0, false);				
			}
		}
	}	
	
	
	public void setPushedAtStartup(boolean pushSceneAtStartup) {
		if (this.pushSceneAtStartup != pushSceneAtStartup) {
			UndoableBooleanEdit edit = new UndoableBooleanEdit(this.pushSceneAtStartup, pushSceneAtStartup) {
				public void setValue(boolean value) {
					setPushedAtStartup(value);
				}
			};
			getUndoManager().addEdit(edit);			
			
			this.pushSceneAtStartup = pushSceneAtStartup;
			firePropertyChange("startupPushChanged", null, new Boolean(pushSceneAtStartup)); //$NON-NLS-1$
		}
	}
	
	public boolean isPushedAtStartup() {
		return pushSceneAtStartup;
	}	
	
	
	public void setSceneMode(final int sceneMode, boolean makeUndoable) {
		if (makeUndoable) {
			if (this.sceneMode != sceneMode) {
				final int oldSceneMode = this.sceneMode;
				getUndoManager().startCompound(Messages.getString("SceneWobProperties.Scene_Type_Edit"));			
				
				
				AbstractUndoableEdit edit = new AbstractUndoableEdit() {
					public void undo() {
						super.undo();
						SceneWobProperties.this.sceneMode = oldSceneMode;
						firePropertyChange("sceneModeChanged", null, new Integer(oldSceneMode));			
					}
	
					public void redo() {
						super.redo();
						SceneWobProperties.this.sceneMode = sceneMode;
						firePropertyChange("sceneModeChanged", null, new Integer(sceneMode));
					}
				};
				getUndoManager().addEdit(edit);		
				
			
				/*UndoableIntegerEdit edit = new UndoableIntegerEdit(this.sceneMode, sceneMode) {
					public void setValue(int value) {
						SceneWobProperties.this.sceneMode = value;
						updateScenePorts();
						fireStateChanged("sceneModeChanged", new Integer(value)); //$NON-NLS-1$					
					}
				};
				if (getProject() != null) // ska kunna anropas i predropped, inget projekt satt d
					getProject().getUndoManager().addEdit(edit);		*/	
				
				this.sceneMode = sceneMode;
				updateSceneMode();
				firePropertyChange("sceneModeChanged", null, new Integer(sceneMode)); //$NON-NLS-1$
				
				getUndoManager().endCompound();	
				// Update the proerties tabs

				//getProject().getUndoManager().discardAllEdits();
			}
		} else {
			this.sceneMode = sceneMode;
		}		
		updatePropertyEditors();
		//_getWobModel().setSelected(true);
	}	

	// OBS: Kan vara true fastn inte virtuell scene, om scenen inte r gfx-operator
	public int getSceneMode() {
		return sceneMode;
	}
	
	public PropertyEditorCollection createPropertyEditorCollection() {
		PropertyEditorCollection propertyEditorCollection = new PropertyEditorCollection(this);

		BooleanPropertyEditor startupPushEditor = new BooleanPropertyEditor("startupPushChanged", Messages.getString("SceneWobProperties.Push_at_Startup"), isPushedAtStartup()); //$NON-NLS-1$ //$NON-NLS-2$
		startupPushEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				boolean value = ((Boolean) evt.getNewValue()).booleanValue();
				setPushedAtStartup(value);
			}
		});
		propertyEditorCollection.addPropertyEditor(startupPushEditor);


		// Create Wob name property editor
		ComboBoxPropertyEditor sceneModeEditor = new ComboBoxPropertyEditor("sceneModeChanged", Messages.getString("SceneWobProperties.Scene_Type"), new String[] {Messages.getString("SceneWobProperties.Offscreen"), Messages.getString("SceneWobProperties.Standard_grouping"), Messages.getString("SceneWobProperties.Buffered_grouping")}, sceneMode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
		sceneModeEditor.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				int type = ((Integer) evt.getNewValue()).intValue();				
				setSceneMode(type, true);
			}
		});
		propertyEditorCollection.addPropertyEditor(sceneModeEditor);		
		
		
		/*if (super.getWobModel().isGfxOperator()) {
			BooleanPropertyEditor virtualSceneEditor = new BooleanPropertyEditor("virtualSceneChanged", Messages.getString("SceneWobProperties.Virtual_Scene"), isVirtualScene()); //$NON-NLS-1$ //$NON-NLS-2$
			virtualSceneEditor.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					boolean value = ((Boolean) evt.getNewValue()).booleanValue();
					setVirtualScene(value);
				}
			});
			propertyEditorCollection.addPropertyEditor(virtualSceneEditor);
		}*/

		
		addPropertyChangeListener(propertyEditorCollection);
		return propertyEditorCollection;
	}

	/*public void destroyPropertyEditorCollection(PropertyEditorCollection propertyEditorCollection) {
		removeNamedChangeListener(propertyEditorCollection);
	}*/



	public SceneWobProperties() {
		super();
	}

	public void writeData(DataOutputStream dos) {
		try {
			dos.writeBoolean(pushSceneAtStartup);
			dos.writeShort(_getWobModel().getDisplayDimension().width);
			dos.writeShort(_getWobModel().getDisplayDimension().height);

			/*if (getSceneMode() == STANDARD_GROUPING_SCENE_MODE) { // behvs isvirtualscene anropas i wobmodel?
				dos.writeInt(-2); // man ser att scenen r virtuell genom
									// scene-frgen
			} else {*/
			dos.writeInt(_getWobModel().getDisplayColor().getRGB());
			//}
			dos.writeShort(getSceneMode());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void init() {
		OperatorAreaModel operatorAreaModel = _getWobModel().getOperatorAreaModel();
		if (operatorAreaModel != null) {
			operatorAreaModel.setToPixelChanger(true);
			operatorAreaModel.setVerticallyResizable(false);
			operatorAreaModel.setHorizontallyResizable(false);
		}
		
		if (!_getWobModel().getInPorts().originalPortExists(0)) { // For old scenes, that don't have the "push scene"/"scene pushed"-ports

			_getWobModel().getInPorts().addOriginalPort(WobHandler.getFastAccessData("scene").
					getInportConfig(0).createPort(_getWobModel(), null));			
			_getWobModel().getOutPorts().addOriginalPort(WobHandler.getFastAccessData("scene").
					getOutportConfig(0).createPort(_getWobModel(), null));	
			
				/*getWobModel().getInPorts().addOriginalPort(new Port(Messages.getString("Port.scene.push_scene.name"), 0, 0,
			  		Messages.getString("Port.scene.push_scene.info"), true, getWobModel().getInPorts())); //$NON-NLS-1$

				getWobModel().getOutPorts().addOriginalPort(new Port(Messages.getString("Port.scene.scene_pushed.name"), 0, 1,
						Messages.getString("Port.scene.scene_pushed.info"), false, getWobModel().getOutPorts())); //$NON-NLS-1$
	
			*/
			
		  //getWobModel().getInPorts().addOriginalPort(new Port(0, 0, true, getWobModel().getInPorts(), "scene.push_scene", null)); //$NON-NLS-1$
		 // getWobModel().getOutPorts().addOriginalPort(new Port(0, 1, false, getWobModel().getOutPorts(), "scene.scene_pushed", null));		   //$NON-NLS-1$

			//getWobModel().getInPorts().addOriginalPort(new Port("Push Scene", 0, 0, "Fires the \"Scene Pushed\" out port", true, getWobModel().getInPorts())); //$NON-NLS-1$ //$NON-NLS-2$
			//getWobModel().getOutPorts().addOriginalPort(new Port("Scene Pushed", 0, 1, "Fires the Scene itself when Push Scene in port is triggered", false, getWobModel().getOutPorts())); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/*class SceneWobPropertiesPanel extends WobPropertiesPanel {
		private JCheckBox pushSceneCheckBox;

		public void init(WobProperties wobProperties) {
			setWobPropertiesPanelClass(SceneWobPropertiesPanel.class);
			JPanel panel = new JPanel();
			setLayout(new FlowLayout());
			add(panel);

			panel.setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.insets = new Insets(5, 5, 5, 5);
			gbc.weightx = 1;
			gbc.weighty = 1;

			pushSceneCheckBox = new JCheckBox(Messages.getString("SceneWobProperties.Push_Scene_at_presentation_startup"), pushSceneAtStartup); //$NON-NLS-1$
			panel.add(pushSceneCheckBox, gbc);
		}

		public void apply() {
			pushSceneAtStartup = pushSceneCheckBox.isSelected();
		}
	}*/
}