/*
 * @(#)ButtonPreview.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.awt.*;
import javax.swing.*;

/**
 * Used for displaying previews of a button in a
 * specific button state. Used in Advanced mode only.
 * 
 */
public class ButtonPreview extends JComponent {

	private Image buttonImage = null;

	private Image labelImage = null;

	/**
	 * Reference back to the ButtonStateConfigurator object
	 */
	private ButtonStateConfigurator configurator;

	/** Constructs a new ButtonPreview
	 * @param configurator
	 */
	public ButtonPreview(ButtonStateConfigurator configurator) {
		this.configurator = configurator;
	}

	public Dimension getButtonDimension() {
		int width = 0;
		if (buttonImage != null)
			width = buttonImage.getWidth(this);
		if (labelImage != null)
			width = Math.max(labelImage.getWidth(this), width);

		int height = 0;
		if (buttonImage != null)
			height = buttonImage.getHeight(this);
		if (labelImage != null)
			height = Math.max(labelImage.getHeight(this), height);

		return new Dimension(width, height);
	}

	public void updatePreview() {
		if (configurator.getIconFile() == null) {
			buttonImage = null;
		} else {
			try {
				buttonImage = java.awt.Toolkit.getDefaultToolkit().createImage(
						configurator.getIconFile().toURI().toURL());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if (configurator.getLabelImageFile() == null) {
			labelImage = null;
		} else {
			try {
				labelImage = java.awt.Toolkit.getDefaultToolkit().createImage(
						configurator.getLabelImageFile().toURI().toURL());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		updatePreferredSize();
	}

	private void updatePreferredSize() {
		int width = Math.max(100, getButtonDimension().width);
		int height = Math.max(50, getButtonDimension().height);
		setPreferredSize(new Dimension(width, height));
		invalidate();
		repaint();
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		repaint();
	}

	public void paint(Graphics g) {
		if (isEnabled() && buttonImage != null) {
			int imageYPos = (getSize().height - buttonImage.getHeight(this)) / 2;
			int imageXPos = (getSize().width - buttonImage.getWidth(this)) / 2;
			g.drawImage(buttonImage, imageXPos, imageYPos, this);

			if (labelImage != null) {
				int labelYPos = (getSize().height - labelImage.getHeight(this)) / 2;
				int labelXPos = (getSize().width - labelImage.getWidth(this)) / 2;
				g.drawImage(labelImage, labelXPos, labelYPos, this);
			}
		}
		super.paint(g);
	}
}