/*
 * @(#)ButtonStateData.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.io.*;

import com.wirefusion.gui.textimage.*;
import com.wirefusion.project.*;

/**
 * Stores a button state configuration. Only used in Advanced mode. 
 */
public class ButtonStateData extends Object implements Externalizable {
	static final long serialVersionUID = -9256525118436653L;

	/**
	 * Referenc back to the button properties object (currently not used)
	 */
	private ButtonProperties buttonProperties;

	/**
	 * Stores the label image configuration of the TextImage object
	 */
	private TextImage labelImageConfig;

	/**
	 * True if this state is enabled
	 */
	private boolean isEnabled;

	/**
	 * Button images of this state are stored under this ID
	 */
	private String buttonImageDependencyId;

	/**
	 * Label images of this state are stored under this ID
	 */
	private String labelImageDependencyId;

	public ButtonStateData() {
	}
	
	public ButtonStateData(ButtonProperties buttonProperties, boolean isEnabled,
			String buttonImageDependencyId, String labelImageDependencyId) {
		this.isEnabled = isEnabled;
		this.buttonProperties = buttonProperties;
		this.buttonImageDependencyId = buttonImageDependencyId;
		this.labelImageDependencyId = labelImageDependencyId;
	}

	public TextImage getLabelImageConfig() {
		return labelImageConfig;
	}

	public void setLabelImageConfig(TextImage labelImageData) {
		this.labelImageConfig = labelImageData;
	}

	public void setEnabled(boolean enable) {
		isEnabled = enable;
	}

	public boolean isEnabled() {
		return isEnabled;
	}

	public File getButtonImageFile() {
		return buttonProperties.getPlayerResourceById(buttonImageDependencyId);
	}

	public File getLabelImageFile() {
		return buttonProperties.getPlayerResourceById(labelImageDependencyId);
	}

	/**
	 * Sets the button image file
	 * 
	 * @param file
	 *          a button image file
	 */
	public void setButtonImageFile(File file) {
		if (file != null) {
			file = buttonProperties.addPlayerResource(file,
					buttonImageDependencyId, true, WobProperties.LOAD_BEFORE_START);
		} else {
			buttonProperties.removePlayerResourceById(buttonImageDependencyId);
		}
	}

	/**
	 * Sets the label image file
	 * 
	 * @param file
	 *          a label image file
	 */
	public void setLabelImageFile(File file) {
		if (file != null) {
			file = buttonProperties.addPlayerResource(file, labelImageDependencyId,
					true, WobProperties.LOAD_BEFORE_START);
		} else {
			buttonProperties.removePlayerResourceById(labelImageDependencyId);
		}
	}

	public void writeExternal(ObjectOutput objectoutput) {
		try {
			ObjectOutputStream oos = (ObjectOutputStream) objectoutput;
			oos.writeObject(buttonProperties);
			oos.writeUTF(""); oos.writeUTF(""); // For backward compatibility
			oos.writeObject(labelImageConfig);
			oos.writeBoolean(isEnabled);
			// -1 is written by new version of ButtonStateData objects (for backwards
			// compatibility)
			oos.writeInt(-1);
			oos.writeUTF(buttonImageDependencyId);
			oos.writeUTF(labelImageDependencyId);
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	public void readExternal(ObjectInput objectinput) {
		try {
			ObjectInputStream ois = (ObjectInputStream) objectinput;
			buttonProperties = (ButtonProperties) ois.readObject();
			ois.readUTF();ois.readUTF(); // For backward compatibility
			labelImageConfig = (TextImage) ois.readObject();
			isEnabled = ois.readBoolean();

			int index = ois.readInt();
			if (index == -1) { // New version
				buttonImageDependencyId = ois.readUTF();
				labelImageDependencyId = ois.readUTF();
			} else { // Old version, reconstruct dependecy ids
				buttonImageDependencyId = "" + (index + 1);
				labelImageDependencyId = "" + (index + 10);
			}
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

}