/*
 * @(#)ButtonTargetAreaView.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.button;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import com.wirefusion.gui.*;
import com.wirefusion.project.*;
import com.wirefusion.utils.*;

public class ButtonTargetAreaView extends TargetAreaView {
	private final static Color lightGray = new Color(0xCCCCCC);

	private final static Color darkGray = new Color(0x666666);

	/**
	 * The cached image speeds up painting process
	 */
	private Image cachedButtonImage;

	private Image cachedLabelImage;
	
	/**
	 * if the image file is changed, its filename if compared to
	 * cachedIconImageFileName, making it possible to update the cache.
	 */
	private File cachedButtonImageFile = null;

	private File cachedLabelImageFile = null;

	/**
	 * Target Area should only be resizable in Basic mode
	 */
	public boolean isHorizontallyResizable() {
		return (((ButtonProperties) getWobProperties()).getMode() == ButtonProperties.BASIC_MODE);
	}

	/**
	 * Target Area should only be resizable in Basic mode
	 */
	public boolean isVerticallyResizable() {
		return isHorizontallyResizable();
	}
	
	/**
	 * Draws a button label
	 * 
	 * @param w button width
	 * @param h button height
	 * @param label label
	 * @param gfx graphics to draw on
	 */
	private void drawText(int w, int h, String label, Graphics gfx) {
		gfx.setFont(new Font("Dialog", Font.BOLD, 12));
		FontMetrics fm = gfx.getFontMetrics();
		fm.stringWidth(label);
		gfx.setColor(Color.black);
		int x = (w - fm.stringWidth(label)) / 2;
		int y = (h - fm.getAscent() - fm.getDescent()) / 2 + fm.getAscent();
		gfx.drawString(label, x, y);
	}

	public void paintContents(Graphics g) {
		ButtonProperties buttonProperties = (ButtonProperties) getWobProperties();
		if (buttonProperties.getMode() == ButtonProperties.BASIC_MODE) {
			String label = buttonProperties.getLabel();
			int w = getSize().width;
			int h = getSize().height;

			g.setColor(lightGray);
			g.fillRect(0, 0, w - 1, h - 1);
			g.setColor(darkGray);
			g.drawRect(0, 0, w - 2, h - 2);
			g.setColor(Color.white);
			g.drawRect(1, 1, w - 2, h - 2);
			drawText(w, h, label, g);
		} else { // Advanced mode
			// Update cached button image if needed
			if (cachedButtonImage == null
					|| !buttonProperties.getButtonUpImageFile().equals(cachedButtonImageFile)) {
				cachedButtonImageFile = buttonProperties.getButtonUpImageFile();
				cachedButtonImage = new ImageIcon(cachedButtonImageFile.getAbsolutePath()).getImage();
			}
			if (buttonProperties.getLabelUpImageFile() != null) {
				if (cachedLabelImage == null
						|| !buttonProperties.getLabelUpImageFile().equals(cachedLabelImageFile)) {
					cachedLabelImageFile = buttonProperties.getLabelUpImageFile();
					cachedLabelImage = new ImageIcon(cachedLabelImageFile.getAbsolutePath()).getImage();
				}
			}

			if (cachedButtonImage != null) {
				int imageYPos = (getSize().height - cachedButtonImage.getHeight(this)) / 2;
				int imageXPos = (getSize().width - cachedButtonImage.getWidth(this)) / 2;
				g.drawImage(cachedButtonImage, imageXPos, imageYPos, this);
			}

			if (buttonProperties.getLabelUpImageFile() != null) {
				if (cachedLabelImage != null) {
					int labelYPos = (getSize().height - cachedLabelImage.getHeight(this)) / 2;
					int labelXPos = (getSize().width - cachedLabelImage.getWidth(this)) / 2;
					g.drawImage(cachedLabelImage, labelXPos, labelYPos, this);
				}
			}
		}
	}
}
