/*
 * @(#)InvertWob.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.invert.player;

import java.awt.*;

import wobs.scene.player.*;
import com.wirefusion.player.*;

/**
 * This filter Wob will invert the pixels inside its Target Area. It has support for
 * an Alpha Channel, which can be sent to its "Set Alpha Channel" in-port.
 */
public class InvertWob extends Wob {
	private static final String ALPHA_NAME = "alpha";
	
	public void prepareForSceneRender() {
		super.prepareForSceneRender();
		SceneWob alphaChannel = (SceneWob)getChild(ALPHA_NAME);		
		if (alphaChannel != null) {
			alphaChannel.render();
		}		
	}
	
	public void filterPixels(WfImage sceneImage, WfImage processedImage, Rectangle targetArea, Rectangle areaToProcess) {
		int[] scenePixels = sceneImage.getPixels();
		int[] processedPixels = processedImage.getPixels();
		
		// Calculate the number of pixels per line 
		int scansize = sceneImage.getWidth();
		// Traverse each pixels inside the visible rectangle
		for (int y = areaToProcess.y; y < areaToProcess.y + areaToProcess.height; y++) {
			int index = areaToProcess.x + y * scansize;
			final int lastIndex = index + areaToProcess.width;
			for (; index < lastIndex; index++) {
				// Calculate the inverted color value for the pixels at position "index"
				// and write it to processedPixels.
				processedPixels[index] = scenePixels[index] ^ 0xffffff;
			}
		}

		// If an alpha channel is found, use it as alpha channel for this filter
		SceneWob alphaChannel = (SceneWob)getChild(ALPHA_NAME);
		if (alphaChannel != null) {
			processedImage.setAlphaFromBlueMask(alphaChannel.getStageImage(), targetArea);
		}
	}
	
	// Listen to in-port events
	public void processInportEvent(Wob arg, int inport) {
		super.processInportEvent(arg, inport);
		if (inport == 0) { // Set Alpha Channel
			// Remove any previous Alpha Channel		
			if (getChild(ALPHA_NAME) != null) {
				removeChild(getChild(ALPHA_NAME));
			}
			arg.setName(ALPHA_NAME);
			addChild(arg);
			repaint();
		}
	}

}