/*
 * @(#)SliderProperties.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.slider;

import java.io.*;

import com.wirefusion.project.*;

public class SliderProperties extends WobProperties implements Externalizable {
	static final long serialVersionUID = 7597038636187615240L;

	private boolean horizontal = true;

	private double minValue = 0;

	private double maxValue = 100;

	private double initValue = 0;

	private double tickSpacing = 0;

	public void init() {
		if (getPlayerResourceById("1") == null) {
			addPlayerResource("horizontalslider.gif", "1", LOAD_BEFORE_START);
			addPlayerResource("horizontalmarker.gif", "2", LOAD_BEFORE_START);
		}
	}

	public File getSliderImageFile() {
		return getPlayerResourceById("1");
	}

	public File getMarkerImageFile() {
		return getPlayerResourceById("2");
	}

	public boolean isHorizontal() {
		return horizontal;
	}

	public void writeExternal(ObjectOutput objectoutput) {
		try {
			ObjectOutputStream objectoutputstream = (ObjectOutputStream) objectoutput;
			super.writeExternal(objectoutputstream);
			objectoutputstream.writeBoolean(horizontal);
			objectoutputstream.writeDouble(minValue);
			objectoutputstream.writeDouble(maxValue);
			objectoutputstream.writeDouble(initValue);
			objectoutputstream.writeDouble(tickSpacing);
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	public void readExternal(ObjectInput objectinput) {
		try {
			ObjectInputStream objectinputstream = (ObjectInputStream) objectinput;
			super.readExternal(objectinputstream);
			horizontal = objectinputstream.readBoolean();
			// For backwards compatibility
			if (Project.getLoadingWobVersion("slider") < 1.2) {
				objectinputstream.readUTF(); //sliderImageFilename
				objectinputstream.readUTF(); //markerImageFilename
			}
			minValue = objectinputstream.readDouble();
			maxValue = objectinputstream.readDouble();
			initValue = objectinputstream.readDouble();
			if (Project.getLoadingWobVersion("slider") >= 1.1)
				tickSpacing = objectinputstream.readDouble();
		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	public SliderProperties() {
	}

	public void writeData(DataOutputStream dataoutputstream) {
		try {
			dataoutputstream.writeBoolean(horizontal);
			dataoutputstream.writeDouble(minValue);
			dataoutputstream.writeDouble(maxValue);
			dataoutputstream.writeDouble(initValue);
			dataoutputstream.writeDouble(tickSpacing);
		} catch (IOException ioexception) {
			ioexception.printStackTrace();
		}
	}

	public double getInitValue() {
		return initValue;
	}

	public void setInitValue(double initValue) {
		this.initValue = initValue;
	}

	public double getMaxValue() {
		return maxValue;
	}

	public void setMaxValue(double maxValue) {
		this.maxValue = maxValue;
	}

	public double getMinValue() {
		return minValue;
	}

	public void setMinValue(double minValue) {
		this.minValue = minValue;
	}

	public double getTickSpacing() {
		return tickSpacing;
	}

	public void setTickSpacing(double tickSpacing) {
		this.tickSpacing = tickSpacing;
	}

	public void setHorizontal(boolean horizontal) {
		this.horizontal = horizontal;
	}

}