/*
 * @(#)SliderPropertiesPanel.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.slider;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import com.wirefusion.gui.*;
import com.wirefusion.gui.misc.*;
import com.wirefusion.project.*;
import com.wirefusion.utils.*;

public class SliderPropertiesPanel extends WobPropertiesPanel implements
		ActionListener {
	private JLabel minLabel;

	private JLabel maxLabel;

	private JLabel tickSpacingLabel;

	private JComboBox orientationComboBox;

	private JButton SliderButton;

	private JButton MarkerButton;

	public NumberTextField minField;

	public NumberTextField maxField;

	public NumberTextField initField;

	public NumberTextField tickSpacingField;

	private File sliderFile;

	private File markerFile;

	private SliderProperties sliderProperties;

	public void init(WobProperties wobProperties) {
		sliderProperties = (SliderProperties) wobProperties;
		JPanel panel = new JPanel();
		setLayout(new FlowLayout());
		add(panel);

		sliderFile = sliderProperties.getPlayerResourceById("1"); 
		markerFile = sliderProperties.getPlayerResourceById("2"); 

		GridBagLayout gridbaglayout = new GridBagLayout();
		GridBagConstraints gridbagconstraints = new GridBagConstraints();
		panel.setLayout(gridbaglayout);
		gridbagconstraints.weightx = 1;
		gridbagconstraints.weighty = 1;
		gridbagconstraints.insets = new Insets(3, 5, 3, 5);

		JLabel orientationLabel = new JLabel(Messages
				.getString("SliderProperties.Orientation"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 0;
		gridbagconstraints.anchor = GridBagConstraints.EAST;
		panel.add(orientationLabel, gridbagconstraints);

		orientationComboBox = new JComboBox();
		orientationComboBox.setEditable(false);
		orientationComboBox.addItem(Messages
				.getString("SliderProperties.Horizontal"));
		orientationComboBox
				.addItem(Messages.getString("SliderProperties.Vertical"));
		orientationComboBox.setSelectedIndex((sliderProperties.isHorizontal()) ? 0
				: 1);
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		gridbagconstraints.gridx = 1;
		panel.add(orientationComboBox, gridbagconstraints);
		orientationComboBox.addActionListener(this);
		orientationComboBox.setActionCommand("orientation");

		JLabel sliderImageLabel = new JLabel(Messages
				.getString("SliderProperties.Slider_image"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 1;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		gridbagconstraints.anchor = GridBagConstraints.EAST;
		panel.add(sliderImageLabel, gridbagconstraints);

		SliderButton = new JButton("...");
		SliderButton.setActionCommand("Slider");
		SliderButton.setPreferredSize(new Dimension(100, SliderButton
				.getPreferredSize().height));
		SliderButton.addActionListener(this);
		gridbagconstraints.anchor = GridBagConstraints.CENTER;
		gridbagconstraints.gridwidth = 1;
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		panel.add(SliderButton, gridbagconstraints);

		JLabel markerImageLabel = new JLabel(Messages
				.getString("SliderProperties.Marker_image"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 2;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		gridbagconstraints.anchor = GridBagConstraints.EAST;
		panel.add(markerImageLabel, gridbagconstraints);

		MarkerButton = new JButton("...");
		MarkerButton.setActionCommand("Marker");
		MarkerButton.setPreferredSize(new Dimension(100, SliderButton
				.getPreferredSize().height));
		MarkerButton.addActionListener(this);
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		panel.add(MarkerButton, gridbagconstraints);

		minLabel = new JLabel(Messages.getString("SliderProperties.Min_value"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 3;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		gridbagconstraints.anchor = GridBagConstraints.EAST;
		panel.add(minLabel, gridbagconstraints);

		minField = new NumberTextField(new Double(sliderProperties.getMinValue()));
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		panel.add(minField, gridbagconstraints);

		maxLabel = new JLabel(Messages.getString("SliderProperties.Max_value"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 4;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		panel.add(maxLabel, gridbagconstraints);

		maxField = new NumberTextField(new Double(sliderProperties.getMaxValue()));
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		gridbaglayout.setConstraints(maxField, gridbagconstraints);
		panel.add(maxField);

		JLabel initLabel = new JLabel(Messages
				.getString("SliderProperties.Initial_value"));
		gridbagconstraints.gridy = 5;
		gridbagconstraints.gridx = 0;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		panel.add(initLabel, gridbagconstraints);

		initField = new NumberTextField(new Double(sliderProperties.getInitValue()));
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		panel.add(initField, gridbagconstraints);

		tickSpacingLabel = new JLabel(Messages
				.getString("SliderProperties.Tick_Spacing"));
		gridbagconstraints.gridx = 0;
		gridbagconstraints.gridy = 6;
		gridbagconstraints.fill = GridBagConstraints.NONE;
		panel.add(tickSpacingLabel, gridbagconstraints);

		tickSpacingField = new NumberTextField(new Double(sliderProperties
				.getTickSpacing()));
		tickSpacingField.setMin(0);
		gridbagconstraints.gridx = 1;
		gridbagconstraints.fill = GridBagConstraints.HORIZONTAL;
		gridbaglayout.setConstraints(tickSpacingField, gridbagconstraints);
		panel.add(tickSpacingField);
	}

	private boolean horizontalSelection() {
		return (orientationComboBox.getSelectedIndex() == 0);
	}

	private File loadFile() {
		File file = null;
		File defaultFile = com.wirefusion.project.WobUtilities.getDefaultFolder(
						"sliderimage", new File(WFConstants.USER_RESOURCES_FOLDER, "images" + File.separator + "sliders" + File.separator));
		ImageFileChooser imagefilechooser = new ImageFileChooser(defaultFile);
		imagefilechooser.setImageFileFilters(new String[] { "gif", "jpg", "png" });
		int i = imagefilechooser.showOpenDialog(getDialog());
		WobUtilities.setDefaultFolder("sliderimage", imagefilechooser.getSelectedFile());
		if (i == 0) {
			file = imagefilechooser.getSelectedFile();
		} else {
			return null;
		}
		return file;
	}

	public void actionPerformed(ActionEvent actionevent) {
		if (actionevent.getActionCommand().equals("Slider")) {
			File file = loadFile();
			if (file != null) {
				sliderFile = file;
			}
		} else if (actionevent.getActionCommand().equals("Marker")) {
			File file = loadFile();
			if (file != null) {
				markerFile = file;
			}
		} else if (actionevent.getActionCommand().equals("orientation")) {
			if (horizontalSelection()) {
				sliderFile = sliderProperties.getFileInWobFolder("horizontalslider.gif");
				markerFile = sliderProperties.getFileInWobFolder("horizontalmarker.gif");
			} else {
				sliderFile = sliderProperties.getFileInWobFolder("verticalslider.gif");
				markerFile = sliderProperties.getFileInWobFolder("verticalmarker.gif");
			}
		}
	}

	public void apply() {
		sliderProperties.setHorizontal(horizontalSelection());
		sliderProperties.setMinValue(minField.getDouble());
		sliderProperties.setMaxValue(maxField.getDouble());
		sliderProperties.setInitValue(initField.getDouble());
		sliderProperties.setTickSpacing(tickSpacingField.getDouble());

		String oldSliderFileName = sliderProperties.getPlayerResourceById("1").getName();
		sliderProperties.addPlayerResource(
				sliderFile, "1", true, WobProperties.LOAD_BEFORE_START);

		String oldMarkerFileName = sliderProperties.getPlayerResourceById("2").getName();
		sliderProperties.addPlayerResource(
				markerFile, "2", true, WobProperties.LOAD_BEFORE_START);

		if (!oldSliderFileName.equals(sliderProperties.getPlayerResourceById("1").getName())
				|| !oldMarkerFileName.equals(sliderProperties.getPlayerResourceById("2").getName())) {
			try {
				Image sliderImg = (new ImageIcon(sliderProperties.getSliderImageFile()
						.toURI().toURL())).getImage();
				Image markerImg = (new ImageIcon(sliderProperties.getMarkerImageFile()
						.toURI().toURL())).getImage();
				int width = Math.max(sliderImg.getWidth(this), markerImg
						.getWidth(WF.mainFrame));
				int height = Math.max(sliderImg.getHeight(this), markerImg
						.getHeight(WF.mainFrame));
				sliderProperties.getTargetArea().setSize(new Dimension(width, height));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public boolean validateInput() {
		return (minField.check() && maxField.check() && initField.check() && tickSpacingField
				.check());
	}
}
