/*
 * @(#)SliderTargetAreaView.java
 *
 * Copyright 2007 Demicron, Inc. All rights reserved.
 *
 * This software is the proprietary information of Demicron, Inc.
 */
package wobs.slider;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import com.wirefusion.gui.*;
import com.wirefusion.project.*;

public class SliderTargetAreaView extends TargetAreaView {
	private Image sliderImg = null;

	private Image markerImg = null;

	private String sliderFileName = null;

	private String markerFileName = null;

	public void updateImages() {
		SliderProperties sliderProperties = (SliderProperties) getWobProperties();
		try {
			if (sliderImg == null
					|| !sliderFileName.equals(sliderProperties.getSliderImageFile()
							.getName())) {
				File sliderFile = sliderProperties.getSliderImageFile();
				sliderFileName = sliderFile.getName();
				sliderImg = (new ImageIcon(sliderFile.toURI().toURL())).getImage();
			}

			if (markerImg == null
					|| !markerFileName.equals(sliderProperties.getMarkerImageFile()
							.getName())) {
				File markerFile = sliderProperties.getMarkerImageFile();
				markerFileName = markerFile.getName();
				markerImg = (new ImageIcon(markerFile.toURI().toURL())).getImage();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean isHorizontallyResizable() {
		return ((SliderProperties) getWobProperties()).isHorizontal();
	}

	public boolean isVerticallyResizable() {
		return !((SliderProperties) getWobProperties())
				.isHorizontal();
	}

	public void paintContents(Graphics g) {
		updateImages();
		SliderProperties sliderProperties = (SliderProperties) getWobProperties();
		Rectangle rect = getBounds();

		int sliderWidth = sliderImg.getWidth(WF.mainFrame);
		int sliderHeight = sliderImg.getHeight(WF.mainFrame);
		int markerWidth = markerImg.getWidth(WF.mainFrame);
		int markerHeight = markerImg.getHeight(WF.mainFrame);
		if (sliderProperties.isHorizontal()) {
			if (rect.width > sliderWidth) {
				Point s1 = new Point(0, 0);
				Point s2 = new Point(sliderWidth / 2, 0);
				Point s3 = new Point(sliderWidth, 0);
				// Point s4 = new Point(s1.x, s1.y + sliderHeight);
				Point s5 = new Point(s2.x, s2.y + sliderHeight);
				Point s6 = new Point(s3.x, s3.y + sliderHeight);

				Point d1 = new Point(0, (rect.height - sliderHeight) / 2);
				Point d2 = new Point(s2.x, d1.y);
				Point d3 = new Point(rect.width - sliderWidth / 2, d1.y);
				// Point d4 = new Point(rect.width, d1.y);
				Point d5 = new Point(0, d1.y + sliderHeight);
				Point d6 = new Point(d2.x, d5.y);
				Point d7 = new Point(d3.x, d5.y);
				Point d8 = new Point(rect.width, d5.y);

				g.drawImage(sliderImg, d1.x, d1.y, d6.x, d6.y, s1.x, s1.y, s5.x, s5.y,
						null);
				g.drawImage(sliderImg, d3.x, d3.y, d8.x, d8.y, s2.x, s2.y, s6.x, s6.y,
						null);
				g.drawImage(sliderImg, d2.x, d2.y, d7.x, d7.y, s2.x, s2.y, s5.x + 1,
						s5.y, null);
			} else {
				Point s1 = new Point(0, 0);
				Point s2 = new Point((sliderWidth - (sliderWidth - rect.width)) / 2, 0);
				Point s3 = new Point(s2.x + (sliderWidth - rect.width), 0);
				Point s4 = new Point(sliderWidth, 0);
				// Point s5 = new Point(0,sliderHeight);
				Point s6 = new Point(s2.x, sliderHeight);
				// Point s7 = new Point(s3.x,sliderHeight);
				Point s8 = new Point(s4.x, sliderHeight);

				Point d1 = new Point(0, (rect.height - sliderHeight) / 2);
				Point d2 = new Point(rect.width / 2, d1.y);
				Point d3 = new Point(rect.width, d1.y);
				Point d4 = new Point(0, d1.y + sliderHeight);
				Point d5 = new Point(d2.x, d4.y);
				Point d6 = new Point(d3.x, d4.y);

				g.drawImage(sliderImg, d1.x, d1.y, d5.x, d5.y, s1.x, s1.y, s6.x, s6.y,
						null);
				g.drawImage(sliderImg, d2.x, d2.y, d6.x, d6.y, s3.x, s3.y, s8.x, s8.y,
						null);
			}
		} else {
			if (rect.height > sliderHeight) {
				Point s1 = new Point(0, 0);
				Point s2 = new Point(0, sliderHeight / 2);
				Point s3 = new Point(0, sliderHeight);
				// Point s4 = new Point(s1.x, s1.y + sliderHeight);
				Point s5 = new Point(s2.x + sliderWidth, s2.y);
				Point s6 = new Point(s3.x + sliderWidth, s3.y);

				Point d1 = new Point((rect.width - sliderWidth) / 2, 0);
				Point d2 = new Point(d1.x, s2.y);
				Point d3 = new Point(d1.x, rect.height - sliderHeight / 2);
				// Point d4 = new Point(rect.width, d1.y);
				Point d5 = new Point(d1.x + sliderWidth, 0);
				Point d6 = new Point(d5.x, d2.y);
				Point d7 = new Point(d5.x, d3.y);
				Point d8 = new Point(d5.x, rect.height);

				g.drawImage(sliderImg, d1.x, d1.y, d6.x, d6.y, s1.x, s1.y, s5.x, s5.y,
						null);
				g.drawImage(sliderImg, d3.x, d3.y, d8.x, d8.y, s2.x, s2.y, s6.x, s6.y,
						null);
				g.drawImage(sliderImg, d2.x, d2.y, d7.x, d7.y, s2.x, s2.y, s5.x,
						s5.y + 1, null);
			} else {
				Point s1 = new Point(0, 0);
				Point s2 = new Point(0,
						(sliderHeight - (sliderHeight - rect.height)) / 2);
				Point s3 = new Point(0, s2.y + (sliderHeight - rect.height));
				Point s4 = new Point(0, sliderHeight);
				// Point s5 = new Point(sliderWidth, 0);
				Point s6 = new Point(sliderWidth, s2.y);
				// Point s7 = new Point(sliderWidth, s3.y);
				Point s8 = new Point(sliderWidth, s4.y);

				Point d1 = new Point((rect.width - sliderWidth) / 2, 0);
				Point d2 = new Point(d1.x, rect.height / 2);
				Point d3 = new Point(d1.x, rect.height);
				Point d4 = new Point(d1.x + sliderWidth, 0);
				Point d5 = new Point(d4.x, d2.y);
				Point d6 = new Point(d4.x, d3.y);

				g.drawImage(sliderImg, d1.x, d1.y, d5.x, d5.y, s1.x, s1.y, s6.x, s6.y,
						null);
				g.drawImage(sliderImg, d2.x, d2.y, d6.x, d6.y, s3.x, s3.y, s8.x, s8.y,
						null);
			}
		}
		int markerx = (rect.width - markerWidth) / 2;
		int markery = (rect.height - markerHeight) / 2;
		g.drawImage(markerImg, markerx, markery, null);

	}
}
